import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class ImageMagnifier extends Applet {
    MediaTracker tracker;
    Image background;
    int width = -1;
    int height = -1;
    boolean ready = false;

    int xo = 0;
    int yo = 0;
    int box_x = 0;
    int box_y = 0;
    int box_width = 100;
    int box_height = 100;

    // The offscreen image
    private Image offscreen;
    private Image backing;
    int off_width = -1;
    int off_height = -1;

    Graphics anim = null;

    //----------------------------------------------------------------

    public void init() {
	tracker = new MediaTracker(this);
        background = getImage(getCodeBase(), "view.png");
	tracker.addImage(background,0);
	prepareImage(background,null);
	anim = getGraphics();

	this.enableEvents(AWTEvent.MOUSE_EVENT_MASK |
			  AWTEvent.MOUSE_MOTION_EVENT_MASK |
			  AWTEvent.KEY_EVENT_MASK |
			  AWTEvent.FOCUS_EVENT_MASK |
			  AWTEvent.COMPONENT_EVENT_MASK |
			  AWTEvent.WINDOW_EVENT_MASK);
	updateState();
      }

    public void updateState() {
	if (!ready) {
	    if (tracker.statusID(0, false) == MediaTracker.COMPLETE) {
		width = background.getWidth(null);
		height = background.getHeight(null);
		ready = true;
		Dimension d = getSize();
		xo = (d.width-width)/2;
		yo = (d.height-height)/2;
		off_width = d.width;
		off_height = d.height;
		offscreen = createImage( off_width, off_height );
		backing = createImage( off_width, off_height );
	
		// Get the off-screen graphics object
		Graphics gg = offscreen.getGraphics();
		
		// Clear the off-screen graphics object
		gg.setColor( getBackground() );
		gg.fillRect( 0, 0, off_width, off_height );
		gg.setColor( getForeground() );
		
		// Draw to the off-screen graphics object
		gg.setColor(Color.lightGray);
		gg.fillRect(0,0,d.width,d.height);
		gg.drawImage(background, xo, yo, this);
		
		// We don't need this Graphics object anymore
		gg.dispose();
	    }
	}
    }

    //----------------------------------------------------------------

    public void processMouseMotionEvent(MouseEvent e) {
	updateState();
	if (ready) {
	    //erase_box();
	    
	    int px = box_x;
	    int py = box_y;

	    box_x = e.getX()-xo;
	    box_y = e.getY()-yo;

	    int nx = box_x;
	    int ny = box_y;

	    Rectangle rp = new Rectangle(xo+px, yo+py, box_width, box_height);
	    Rectangle rn = new Rectangle(xo+nx, yo+ny, box_width, box_height);
	    Rectangle ru = new Rectangle();

	    Rectangle.union(rp,rn,ru);

	    if (px<nx) {
		anim.drawImage(offscreen, 
			       ru.x,ru.y,ru.x+(nx-px),ru.y+ru.height,
			       ru.x,ru.y,ru.x+(nx-px),ru.y+ru.height,
			       null );
		
	    } else {
		anim.drawImage(offscreen, 
			       ru.x+rn.width,ru.y,ru.x+rn.width+(px-nx),ru.y+ru.height,
			       ru.x+rn.width,ru.y,ru.x+rn.width+(px-nx),ru.y+ru.height,
			       null );
	    }
	    if (py<ny) {
		anim.drawImage(offscreen, 
			       ru.x,ru.y,ru.x+ru.width,ru.y+(ny-py),
			       ru.x,ru.y,ru.x+ru.width,ru.y+(ny-py),
			       null );
		
	    } else {
		anim.drawImage(offscreen, 
			       ru.x,ru.y+rn.height,ru.x+ru.width,ru.y+rn.height+(py-ny),
			       ru.x,ru.y+rn.height,ru.x+ru.width,ru.y+rn.height+(py-ny),
			       null );
	    }

	    /*
	    anim.drawImage(offscreen, 
			   xo+px, yo+py, xo+px+box_width, yo+py+box_height, 
			   xo+px, yo+py, xo+px+box_width, yo+py+box_height, 
			   null );
	    */

	    draw_box(anim);

	    //repaint();
	}
    }

    //----------------------------------------------------------------

    void draw_box(Graphics g)
      {
        Graphics g2 = backing.getGraphics();

        g2.clipRect(box_x+xo, box_y+yo, box_width, box_height);
	
	g2.drawImage(offscreen, 
		     xo+box_x,yo+box_y,xo+box_x+box_width,yo+box_y+box_height, 
		     xo+box_x,yo+box_y,xo+box_x+box_width,yo+box_y+box_height, 
		     null );
        g2.drawImage(background,
                     xo-box_x, yo-box_y,
                     width*2, height*2, null);
	g.drawImage(backing,
		    xo+box_x,yo+box_y,xo+box_x+box_width,yo+box_y+box_height, 
		    xo+box_x,yo+box_y,xo+box_x+box_width,yo+box_y+box_height, 
		    null );
        g.setColor(Color.red);
        g.drawRect(xo+box_x, yo+box_y, box_width-1, box_height-1);
	g2.dispose();
      }

    //----------------------------------------------------------------

    public void paint(Graphics g) {
	updateState();
	Dimension d = getSize();
        g.setColor(Color.lightGray);
	g.fillRect(0,0,d.width,d.height);
	g.drawImage(background, xo, yo, this);
	if (ready) {
	    draw_box(g);
	}
    }
}
